# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,persistence)
S3method(as.matrix,persistence)
S3method(as_diagram,PHom)
S3method(as_diagram,persistence)
S3method(as_persistence,PHom)
S3method(as_persistence,data.frame)
S3method(as_persistence,diagram)
S3method(as_persistence,hclust)
S3method(as_persistence,list)
S3method(as_persistence,matrix)
S3method(as_persistence,persistence)
S3method(format,persistence)
S3method(format,persistence_set)
S3method(print,persistence)
S3method(print,persistence_set)
export(as_diagram)
export(as_persistence)
export(as_persistence_set)
export(bottleneck_distance)
export(bottleneck_pairwise_distances)
export(get_pairs)
export(kantorovich_distance)
export(kantorovich_pairwise_distances)
export(wasserstein_distance)
export(wasserstein_pairwise_distances)
useDynLib(phutil, .registration = TRUE)
